/*
  Multi Sensor Receiver Reference Design

  This code provides a reference design for the Radio Bridge
  Particle gateway to receive sensor inputs from the SubGig, 345MHz,
  and 319MHz BridgeBee modules.

  (c) 2017 Radio Bridge Inc. and its subsidiaries.
*/

#include "application.h"
#include "serial-comm.h"
#include "message-processing.h"
#include "utils.h"

// there is a data structure for sensor state maintained for
// every sensor ID that arrives
SENSOR_DATA sensor_data[MAX_SENSORS];

// buffers for receive serial data
extern uint8_t rx_data1[MAX_PACKET_SIZE];
extern uint8_t rx_data2[MAX_PACKET_SIZE];
extern uint8_t rx_data3[MAX_PACKET_SIZE];

// parse the sensor data received from the SubGig BridgeBee
uint8_t process_message_subgig(void)
{

    // assemble the sensor ID from 4 bytes in the message
    uint32_t sensor_id;

    // index of sensor in our data structure array
    uint8_t sensor_index;


    // assemble sensor ID based on 4 bytes in message
    sensor_id = ascii_long_to_hex(&rx_data1[4]);

    // find the index of the sensor we just received
    sensor_index = find_sensor_index(sensor_id);

    // if the sensor is not found then create an entry
    if(sensor_index == INVALID_SENSOR_INDEX)
        sensor_index = add_sensor(sensor_id);

    // take the appropriate bits from the serial packet and add to data
    // structure for this sensor
    sensor_data[sensor_index].tamper = (rx_data1[12]>>1) & 0x01;
    sensor_data[sensor_index].input1 = (rx_data1[12]) & 0x01;

    return sensor_index;

}


// parse the sensor data received from the 345MHz BridgeBee
uint8_t process_message_345(void)
{

    // assemble the sensor ID from 3 bytes in the message
    uint32_t sensor_id;

    // index of sensor in our data structure array
    uint8_t sensor_index;

    uint8_t sensor_bits = 0;

    // assemble sensor ID based on 3 bytes in message
    sensor_id = ascii_long_to_hex(&rx_data2[4]);

    // 319 message is only 24 bits so discard lower bits
    sensor_id = sensor_id >> 8;

    // find the index of the sensor we just received
    sensor_index = find_sensor_index(sensor_id);

    // if the sensor is not found then create an entry
    if(sensor_index == INVALID_SENSOR_INDEX)
        sensor_index = add_sensor(sensor_id);

    // convert the sensor data from ascii to raw bits
    sensor_bits = ascii_nibble_to_hex(rx_data2[11]);

    // take the appropriate bits from the serial packet and add to data
    // structure for this sensor

    sensor_data[sensor_index].tamper = (sensor_bits>>2) & 0x01;

    // open/close or motion
    sensor_data[sensor_index].input1 = ((sensor_bits>>1) & 0x01) || ((sensor_bits>>3) & 0x01);


    return sensor_index;

}


// parse the sensor data received from the 319MHz BridgeBee
uint8_t process_message_319(void)
{
    // assemble the sensor ID from 3 bytes in the message
    uint32_t sensor_id;

    // index of sensor in our data structure array
    uint8_t sensor_index;

    uint8_t sensor_bits = 0;

    // assemble sensor ID based on 3 bytes in message
    sensor_id = ascii_long_to_hex(&rx_data3[4]);

    // 319 message is only 24 bits so discard lower bits
    sensor_id = sensor_id >> 8;

    // find the index of the sensor we just received
    sensor_index = find_sensor_index(sensor_id);

    // if the sensor is not found then create an entry
    if(sensor_index == INVALID_SENSOR_INDEX)
        sensor_index = add_sensor(sensor_id);

    // take the appropriate bits from the serial packet and add to data
    // structure for this sensor

    // convert the sensor data from ascii to raw bits
    sensor_bits = ascii_nibble_to_hex(rx_data3[18]);
    sensor_data[sensor_index].tamper = (sensor_bits>>1) & 0x01;

    sensor_bits = ascii_nibble_to_hex(rx_data3[17]);
    sensor_data[sensor_index].input1 = (sensor_bits>>1) & 0x01;

    return sensor_index;

}


// find the index of the incoming sensor
uint8_t find_sensor_index(uint32_t sensor_id)
{
    uint8_t i;

    // look for the sensor ID in the list and return index if found
    for(i=0;i<MAX_SENSORS;i++)
    {
        if(sensor_data[i].sensor_id == sensor_id)
            return i;
    }

    // not founds so return invalid index
    return INVALID_SENSOR_INDEX;
}

// find the index of the next available slot in the sensor list
uint8_t find_next_available_sensor_index()
{
    uint8_t i;

    // search for the next slot that contains a 0 for sensor id
    // which indicates an available slot
    for(i=0;i<MAX_SENSORS;i++)
    {
        if(sensor_data[i].sensor_id == 0)
            return i;
    }

    // no slots available so return invalid index
    return INVALID_SENSOR_INDEX;
}


// initialize the sensor list
void init_sensor_list(void)
{
    uint8_t i;

    // empty all slots for storing sensors
    for(i=0;i<MAX_SENSORS;i++)
    {
        sensor_data[i].sensor_id = 0;
    }

}


// add the new sensor to the list
uint8_t add_sensor(uint32_t sensor_id)
{
    uint8_t new_sensor_index;

    // find the next unused index in sensor list
    new_sensor_index = find_next_available_sensor_index();

    // assign an ID to the new sensor entry
    if(new_sensor_index != INVALID_SENSOR_INDEX)
      sensor_data[new_sensor_index].sensor_id = sensor_id;

    return new_sensor_index;
}
