/*
  Multi Sensor Receiver Reference Design

  This code provides a reference design for the Radio Bridge
  Particle gateway to receive sensor inputs from the SubGig, 345MHz,
  and 319MHz BridgeBee modules.

  (c) 2017 Radio Bridge Inc. and its subsidiaries.
*/

#include "application.h"
#include "message-processing.h"
#include "web-comm.h"
#include "serial-comm.h"
#include "timers.h"
#include "utils.h"
#include "leds.h"

// serial transmit/receive objects for BridgeBee 1
extern SerialTransfer Serial_Receive1;
extern SerialTransfer Serial_Transmit1;

// serial transmit/receive objects for BridgeBee 2
extern SerialTransfer Serial_Receive2;
extern SerialTransfer Serial_Transmit2;

// serial transmit/receive objects for BridgeBee 3
extern SerialTransfer Serial_Receive3;
extern SerialTransfer Serial_Transmit3;

// an array of data structures that contain sensor status
// for each sensor that is reporting
extern SENSOR_DATA sensor_data[MAX_SENSORS];


void setup()
{
  // initialize leds on gateway
  init_leds();

  // initialize serial ports to each BridgeBee module
  init_serial();

  // initialize the 100ms timer
  init_timers();

  // initialize the sensor list
  init_sensor_list();

  // register the variable to be published to the Particle cloud
  register_publish_variables();

  // Debug message for Particle IDE
  Serial.println("just reset");

}


// main loop that manages data flow between BridgeBee devices
// and Particle cloud
void loop()
{

  uint8_t sensor_index;

  // if we received a valid packet from the SubGig BridgeBee (socket 1)
 if(Serial_Receive1.receiveData())
 {
   // process the message reporting on BridgeBee 1 and return the index
   // where that sensor's data is stored
   sensor_index = process_message_subgig();

   // if we received a valid new sensor event then assert LED and publish
   if(sensor_index != INVALID_SENSOR_INDEX)
   {
     // assert the LED for BridgeBee 1
     start_led_flash(LED1);

     // publish sensor data to Particle cloud
     publish_sensor_data(sensor_index);
   }
 }

  // if we received a valid packet from the 345MHz BridgeBee (socket 2)
  if(Serial_Receive2.receiveData())
  {
    // process the message reporting on BridgeBee 2 and return the index
    // where that sensor's data is stored
    sensor_index = process_message_345();

    // if we received a valid new sensor event then assert LED and publish
    if(sensor_index != INVALID_SENSOR_INDEX)
    {
      // assert the LED for BridgeBee 2
      start_led_flash(LED2);

      // publish sensor data to Particle cloud
      publish_sensor_data(sensor_index);
    }
  }

  // if we received a valid packet from the 319MHz BridgeBee (socket 3)
  if(Serial_Receive3.receiveData())
  {
    // process the message reporting on BridgeBee 3 and return the index
    // where that sensor's data is stored
   sensor_index = process_message_319();

   // if we received a valid new sensor event then assert LED and publish
   if(sensor_index != INVALID_SENSOR_INDEX)
   {
     // assert the LED for BridgeBee 3
     start_led_flash(LED3);

     // publish sensor data to Particle cloud
     publish_sensor_data(sensor_index);
   }
  }


  // update timers and check for callback
  update_timers();

  // check running timer events
  check_timers();

}
