/*
  Multi Sensor Receiver Reference Design

  This code provides a reference design for the Radio Bridge
  Particle gateway to receive sensor inputs from the SubGig, 345MHz,
  and 319MHz BridgeBee modules.

  (c) 2017 Radio Bridge Inc. and its subsidiaries.
*/


#include "application.h"
#include "utils.h"


// convert a value coded as an ascii string to raw number
uint8_t ascii_byte_to_hex(uint8_t *asciival)
{
	return (ascii_nibble_to_hex(asciival[0]) << 4) + ascii_nibble_to_hex(asciival[1]);
}

// convert an ascii coded nibble to raw hex value
uint8_t ascii_nibble_to_hex(uint8_t asciival)
{

	if(is_ascii_coded_hex_number(asciival))
		return asciival - 0x30;
	else if(is_ascii_coded_hex_uppercase(asciival))
		return asciival - 0x37;
	else if(is_ascii_coded_hex_lowercase(asciival))
		return asciival - 0x57;
	else
		return 0;
}

// convert byte to ascii string
void hex_to_ascii_byte(uint8_t hex_byte, uint8_t *ascii_val)
{
	uint8_t upper, lower;

	upper = (hex_byte >> 4) & 0x0f;
	lower = hex_byte & 0x0f;

	if(upper < 0x0a)
		ascii_val[0] = upper + 0x30;
	else
		ascii_val[0] = upper + 0x37;

	if(lower < 0x0a)
		ascii_val[1] = lower + 0x30;
	else
		ascii_val[1] = lower + 0x37;

}

// convert a value coded as an ascii string to raw number
uint32_t ascii_long_to_hex(uint8_t *asciival)
{
	return 	(ascii_byte_to_hex(&asciival[0]) << 24) +
			(ascii_byte_to_hex(&asciival[2]) << 16) +
			(ascii_byte_to_hex(&asciival[4]) << 8) +
			 ascii_byte_to_hex(&asciival[6]);
}

// convert a 16-bit value coded as an ascii string to raw number
uint16_t ascii_16_to_hex(uint8_t *asciival)
{
	return 	(ascii_byte_to_hex(&asciival[0]) << 8) +
			 ascii_byte_to_hex(&asciival[2]);
}


// convert long to ascii string
void hex_to_ascii_32(uint32_t hex_byte, uint8_t *ascii_val)
{
	uint8_t next_byte = (hex_byte >> 24) & 0xff;
	hex_to_ascii_byte(next_byte, &ascii_val[0]);

	next_byte = (hex_byte >> 16) & 0xff;
	hex_to_ascii_byte(next_byte, &ascii_val[2]);

	next_byte = (hex_byte >> 8) & 0xff;
	hex_to_ascii_byte(next_byte, &ascii_val[4]);

	next_byte = hex_byte & 0xff;
	hex_to_ascii_byte(hex_byte & 0xff, &ascii_val[6]);

}

// convert 16 bits to ascii string
void hex_to_ascii_16(uint16_t hex_byte, uint8_t *ascii_val)
{
	uint8_t next_byte = (hex_byte >> 8) & 0xff;
	hex_to_ascii_byte(next_byte, &ascii_val[0]);

	next_byte = hex_byte & 0xff;
	hex_to_ascii_byte(hex_byte & 0xff, &ascii_val[2]);
}

// if char is uppercase, convert to lowercase
uint8_t lowercase(uint8_t newchar)
{
	// if uppercase value, return lowercase version
	if((newchar > 0x40) && (newchar < 0x5b))
		return (uint8_t)(newchar + 0x20);
	else
		return newchar;

}

// if char is lowercase, convert to uppercase
uint8_t uppercase(uint8_t newchar)
{
	// if lowercase value, return uppercase version
	if((newchar > 0x60) && (newchar < 0x7b))
		return (uint8_t)(newchar - 0x20);
	else
		return newchar;

}

// convert a string of ascii coded hex chars to uppercase
void ascii_hex_string_to_upper(uint8_t *ascii_hex_string, uint8_t length)
{
	uint8_t i;

	for(i=0;i<length;i++)
		ascii_hex_string[i] = uppercase(ascii_hex_string[i]);
}

// return true if this is an ascii coded hex string. A-F must be uppercase
bool is_ascii_coded_hex_string(uint8_t *ascii_hex_string, uint8_t length)
{
	uint8_t i;

	for(i=0;i<length;i++)
		if(!is_ascii_coded_hex_char(ascii_hex_string[i]))
			return false;

	return true;
}

// return true if this is an ascii coded hex character
bool is_ascii_coded_hex_char(uint8_t ascii_hex_char)
{
	// if an ascii number, lower case, or upper case
	if(is_ascii_coded_hex_number(ascii_hex_char) ||is_ascii_coded_hex_uppercase(ascii_hex_char) || is_ascii_coded_hex_lowercase(ascii_hex_char))
		return true;
	else
		return false;

}

// return true if it is ascii 0-9
bool is_ascii_coded_hex_number(uint8_t ascii_hex_char)
{
	// if an ascii number, lower case, or upper case
	if((ascii_hex_char > 0x2f) && (ascii_hex_char < 0x3a))
		return true;
	else
		return false;

}

// return true if it is ascii A-Z
bool is_ascii_coded_hex_uppercase(uint8_t ascii_hex_char)
{
	// if an ascii number, lower case, or upper case
	if((ascii_hex_char > 0x40) && (ascii_hex_char < 0x47))
		return true;
	else
		return false;

}

// return true if it is ascii a-z
bool is_ascii_coded_hex_lowercase(uint8_t ascii_hex_char)
{
	// if an ascii number, lower case, or upper case
	if((ascii_hex_char > 0x60) && (ascii_hex_char < 0x67))
		return true;
	else
		return false;

}
